#ifndef ARKI_SEGMENT_METADATA_H
#define ARKI_SEGMENT_METADATA_H

#include <arki/types/fwd.h>
#include <arki/segment/fwd.h>
#include <arki/segment.h>
#include <arki/metadata/collection.h>

namespace arki::segment::metadata {

class Index
{
    const Segment& segment;
    std::filesystem::path md_path;

public:
    explicit Index(const Segment& segment);

    bool read_all(std::shared_ptr<arki::segment::data::Reader> reader, metadata_dest_func dest);
    arki::metadata::Collection query_data(const Matcher& matcher, std::shared_ptr<arki::segment::data::Reader> reader);
    void query_summary(const Matcher& matcher, Summary& summary);
};

class Reader : public segment::Reader
{
    Index index;

public:
    Reader(std::shared_ptr<const Segment> segment, std::shared_ptr<const core::ReadLock> lock);
    ~Reader();

    bool read_all(metadata_dest_func dest) override;
    bool query_data(const query::Data& q, metadata_dest_func dest) override;
    void query_summary(const Matcher& matcher, Summary& summary) override;
};

class Writer : public segment::Writer
{
protected:
    arki::metadata::Collection mds;
    Summary sum;

    void add(const Metadata& md, const types::source::Blob& source);
    void write_metadata();

public:
    Writer(std::shared_ptr<const Segment> segment, std::shared_ptr<core::AppendLock> lock);
    ~Writer();

    AcquireResult acquire(arki::metadata::InboundBatch& batch, const WriterConfig& config) override;
};

class Checker : public segment::Checker
{
public:
    using segment::Checker::Checker;

    arki::metadata::Collection scan() override;
    FsckResult fsck(segment::Reporter& reporter, bool quick=true) override;

    std::shared_ptr<segment::Fixer> fixer() override;
};

class Fixer : public segment::Fixer
{
public:
    using segment::Fixer::Fixer;

    MarkRemovedResult mark_removed(const std::set<uint64_t>& offsets) override;
    ReorderResult reorder(arki::metadata::Collection& mds, const segment::data::RepackConfig& repack_config) override;
    size_t remove(bool with_data) override;
    ConvertResult tar() override;
    ConvertResult zip() override;
    ConvertResult compress(unsigned groupsize) override;
    void reindex(arki::metadata::Collection& mds) override;
    void move(std::shared_ptr<arki::Segment> dest) override;
    void test_touch_contents(time_t timestamp) override;
    void test_mark_all_removed() override;
    void test_make_overlap(unsigned overlap_size, unsigned data_idx=1) override;
    void test_make_hole(unsigned hole_size, unsigned data_idx=0) override;
};

}

#endif
